-module(icfp_read).

-export([file/1,file/2]).
-export([test/1]).

-import(lists, [reverse/1]).

-include("icfp.hrl").

test(1) -> file("example.txt");
test(2) -> file("small.txt").

%% file(FileName) -> [Struct].
%% file(FileName, InStruct) -> [Struct].

file(F) -> file(F, #s{}).

file(F, State) ->
    {ok, B} = file:read_file(F),
    scan(B, [State], []).

%% scan(InBinary, [State], [Struct]) -> [Struct].

scan(<< "<B>",T/binary >>, Stack=[S|_], L) ->
    scan(T, [S#s{b=true}|Stack], L);
scan(<< "<EM>",T/binary >>, Stack=[S|_], L) ->
    case S#s.s of
	true -> scan(T, [S|Stack], L);
	false -> scan(T, [S#s{em=not (S#s.em)}|Stack], L)
    end;
scan(<< "<I>",T/binary >>, Stack=[S|_], L) ->
    scan(T, [S#s{i=true}|Stack], L);
scan(<< "<PL>",T/binary >>, Stack=[S|_], L) ->
    scan(T, [S#s{u=0,s=false,em=false,i=false,b=false,tt=false}|Stack], L);
scan(<< "<S>",T/binary >>, Stack=[S|_], L) ->
    scan(T, [S#s{s=true,em=false}|Stack], L);
scan(<< "<TT>",T/binary >>, Stack=[S|_], L) ->
    scan(T, [S#s{tt=true}|Stack], L);
scan(<< "<U>",T/binary >>, Stack=[S|_], L) ->
    case S#s.u of
	N when N < 3 ->
	    scan(T, [S#s{u=N+1}|Stack], L);
	_ ->
	    scan(T, [S|Stack], L)
    end;
scan(<< $<,N:8,$>,T/binary >>, Stack=[S|_], L) when $0 =< N, N =< $9 ->
    scan(T, [S#s{size=N-$0}|Stack], L);
scan(<< $<,C:8,$>,T/binary >>, Stack=[S|_], L) ->
    %% this must be a color code
    scan(T, [S#s{color=list_to_atom([C])}|Stack], L);
scan(<< $<,$/,C:8,$>,T/binary >>, [_|Stack], L) ->
    scan(T, Stack, L);
scan(<< $<,$/,_:8,_:8,$>,T/binary >>, [_|Stack], L) ->
    scan(T, Stack, L);
scan(<< _,_/binary >> = B0, Stack=[S|_], L) ->
    N = collect_chars(B0, 0),
    {Chars,B1} = split_binary(B0, N),
    scan(B1, Stack, [{S,Chars}|L]);
scan(<< >>, _, L) ->
    reverse(L).

collect_chars(<< $<,T/binary >>, N) -> N;
collect_chars(<< _:8,T/binary >>, N) -> collect_chars(T, N+1);
collect_chars(<< >>, N) -> N.



